// RELATED SOFTWARE: Scilab >= 5.5.0, Scilab 6.0
// PACKAGE  : plotplots : plots curves in a piecewise way, with a broken or multiscaled axis
// VERSION  : 1.0 - 2018-
// VERSION IN PROGRESS : 1.0
// AUTHOR   : Samuel GOUGEON - private individual
// COPYRIGHT: Samuel GOUGEON -  2018
// LICENSE for uman : CeCILL : http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt 
//
// DISCLAIMER: This toolbox is provided as is, with NO WARRANTY of any kind.
// Use it to your own risks & responsability.
// Plots piece-wise

// TODO: 
// * interspaces et calage de la grille de l'axe continu
// * intervals bounds must be reached (+ 1 point beyond)
// * Aide en français

function [hcurves, Axes] = plotplots(x0, y0, intervals, cutaxis, varargin)

    // INITIALIZATION
    // --------------
    vers = getversion("scilab");
    vers = vers(1)*10 + vers(2);
    fname = "plotplots"
    nc = size(y0,2)     // number of curves
    x0 = x0(:)
    if min(size(y0))==1 then
        y0 = y0(:)
    end

    // ARGUMENTS ANALYSIS
    // ------------------
    if ~isdef("cutaxis", "l") then
        cutaxis = "x"
    end
    if modulo(length(varargin),2)~=0
        msg = "%s: each option name must be followed by its value\n";
        error(msprintf(msg,fname))
    end
    for i = 1:2:length(varargin)
        v = varargin(i)
        if typeof(v)~="string" | size(v,"*")>1 | ~or(v==["ticksmode" "widths"])
            msg = "%s: Argument #%d: one string among {%s} expected\n";
            error(msprintf(msg,fname, 3+2*i, """ticksmode"",""widths"""));
        else
            execstr(v + "=varargin(i+1)")
        end
    end
    if ~isdef("ticksmode", "l") then
        ticksmode = ""
    end
    if ~isdef("widths", "l") then
        widths = "equal"
    else
        if type(widths)==1
            if length(widths)~=size(intervals,1)
                msg = "%s: widths value: %d components expected\n";
                error(msprintf(msg, fname, size(intervals,1)));
            end
            widths(1) = -abs(widths(1))  // First section has never heading interspace
        end
    end

    // Filtering data for min and max of x and y
    // -----------------------------------------
    ksel = []
    if cutaxis=="x" then
        for intv = intervals'
            ksel = [ksel find(x0>=min(intv) & x0<=max(intv))]
        end
        // ksel = unique([ksel, ksel-1, ksel+1]) // keeping segments crossing axis bounds
        // if ksel(1)==0, ksel(1) = [], end
        // if ksel($)>length(x0), ksel($) = [], end
        ksel = unique(ksel)
        x = x0(ksel)
        y = y0(ksel,:)

    else    // "y" breaks
        for intv = intervals'
            ksel = [ksel find(y0>=min(intv) & y0<=max(intv))]
        end
        ksel = unique(ksel)
        xsel = ind2sub(size(y0), ksel)
        x = x0(xsel)
        y = y0(ksel)
    end
    // min & max
    xmin = min(x)
    xmax = max(x)
    ymin = min(y)
    ymax = max(y)

    drawlater()

    // Plotting the common axes
    ax0 = gca()
    ax0.visible = "off"
    plot2d([],[],0, "011"," ", [xmin ymin xmax ymax])

    if cutaxis=="x"
        ax0.axes_visible(1) = "off"
        ax0.grid = [-1 color("grey70")]     // y-grid only
        ylabel("A","fontsize",2) // to enlarge the margin to anticipate a true legend
        margins = ax0.margins;
        ylabel(".")
    else
        ax0.axes_visible(2) = "off"
        ax0.grid = [color("grey70") -1]     // x-grid only
        xlabel("A","fontsize",2) // to enlarge the margin to anticipate a true legend
        margins = ax0.margins;
        xlabel(".")
    end
    ax0.box = "off"
    ax0.filled = "off"
    if vers > 54 then
        ax0.grid_style = [8 8]
    end

    // Setting axes_bounds of subaxes
    ax0b = ax0.axes_bounds  // all must be plotted in this area
        // The relative cumulated separating space rcss is such that 0.05 <= rcss <= 0.1 
    na = size(intervals,1)
    if typeof(widths)=="string" & part(widths,1)=="-" then
        rcss = 0;
        rss = 0;
    else
        if type(widths)==1
            nas = sum(widths(2:$)<0)
        else
            nas = na-1
        end
        if na<11
            rss = 0.01
            rcss = nas*rss
        else
            rcss = 0.1
            rss = rcss / nas
        end
    end
        //
    if widths=="isoscaled" then
        Dxy = abs(intervals(:,2)-intervals(:,1))  // absolute widths of selected intervals
    elseif type(widths)==1  // values
        Dxy = abs(widths(:))
    else                    // equal
        Dxy = ones(na,1)
    end
    Dxyr = Dxy/sum(Dxy) // *(1-rcss) A VOIR  // relative widths ...
    if cutaxis=="x" then
        wt = ax0b(3)*(1-sum(margins(1:2)))    // Total width
        x_left = ax0b(1) + ax0b(3) * margins(1) // initial left margin
        y_top = ax0b(2)  // Same vertical position for all subaxes
        h = ax0b(4)      // Same height
    else
        x_left = ax0b(1) // same x position for all subaxes
        w = ax0b(3)      // Same width for all subaxes
        ht = ax0b(4)*(1-sum(margins(3:4)))    // Total height
        // initial top margin:
        y_top  = ax0b(2) + ax0b(4) * (1 - margins(4)) + rss*ht
    end

    hcurves = list([],[])
    Axes = []
    iba = 1     // index of broken axis: "x" => 1,  "y" => 2
    if cutaxis ~= "x" then
        iba = 2
    end
    for ia = 1:na
        interv = intervals(ia,:);
        ax_i = newaxes()
        ax_i.visible = "off"
        ax_i.margins(2*iba+(-1:0)) = 0
        ax_i.tight_limits = "on"
        if vers > 54 then
            ax_i.grid_style = [8 8];
        end
        ax_i.grid(iba) = color("grey70")
        ax_i.filled = "off"
        ax_i.clip_state = "off"     // to not clip datatips set on edges
        if cutaxis=="x"
            ax_i.axes_bounds = [x_left, y_top, Dxyr(ia)*wt, h]

            if (type(widths)==10 & part(widths,1)=="-") | ..
                (type(widths)==1 & ia<na & widths(ia+1)<0) then
                tmp = 0
            else
                tmp = rss
            end
            x_left = x_left + (Dxyr(ia)+tmp)*wt

            k = find(x0>=min(interv) & x0<=max(interv))
            plot(x0(k), y0(k,:))
            ax_i.data_bounds(:,2) = [ymin; ymax]
            if ticksmode=="alt" & modulo(ia,2)==0
                ax_i.x_location = "top"
            elseif ticksmode=="shift" & modulo(ia,2)==0
                dyr = 0.025
                ax_i.axes_bounds(4) = h*(1+dyr)            
                ax_i.data_bounds(:,2) = [ymin-(ymax-ymin)*dyr; ymax]
            end
            if interv(1)>interv(2) then
                ax_i.axes_reverse(1) = "on";
            end
        else
            if (type(widths)==10 & part(widths,1)=="-") | ..
                (type(widths)==1 & ia<=na & widths(ia)<0) then
                tmp = 0
            else
                tmp = rss
            end
            y_top = y_top - (Dxyr(ia)+tmp)*ht
            ax_i.axes_bounds = [x_left, y_top, w, Dxyr(ia)*ht]

//            for ic = 1:nc
//                k = find(y0(:,ic)>=min(interv) & y0(:,ic)<=max(interv))
//                k = unique([k, k-1, k+1])
//                //gsort(k,
//                k(find(k<1)) = []
//                k(find(k>size(y0,1))) = []
//                disp(size(k))
//                plot(x0(k), y0(k,:))
//            end
            plot(x0,y0)
            ax_i.data_bounds = [xmin xmax ; min(interv) max(interv)]'
            //ax_i.data_bounds(:,1) = [xmin ; xmax]
            if ticksmode=="alt" & modulo(ia,2)==0
                ax_i.y_location = "right"
            elseif ticksmode=="shift" & modulo(ia,2)==0
                dxr = 0.025
                ax_i.axes_bounds(3) = w*(1+dxr)
                ax_i.axes_bounds(1) = ax_i.axes_bounds(1) - w*dxr
                ax_i.data_bounds(:,1) = [xmin-(xmax-xmin)*dxr; xmax]
            end
            if interv(1)>interv(2) then
                ax_i.axes_reverse(2) = "on";
            end
        end
        tmp = gce()
        for ic = 1:nc
            hcurves(nc-ic+1) = [hcurves(nc-ic+1) tmp.children(ic)]
        end
        ax_i.box = "off";
        ax_i.axes_visible = ["off" "off" "off"]
        ax_i.axes_visible(iba) = "on"
        Axes = [Axes ax_i]
    end

    Axes = [Axes ax0]
    Axes.visible = "on";
    Axes.children.visible = "on";
    sca(ax0);
    drawnow()
endfunction
